#!/bin/sh

APP_NAME="nodered"

ACTION=${1}
shift 1

get_image() {
  do_install_detail
}

do_install_detail() {
  local config port IMAGE_NAME username password LAN_IP
  config=$(uci get nodered.@nodered[0].config_path 2>/dev/null)
  port=$(uci get nodered.@nodered[0].port 2>/dev/null)
  IMAGE_NAME=$(uci get nodered.@nodered[0].image_name 2>/dev/null)
  username=$(uci get nodered.@nodered[0].username 2>/dev/null)
  password=$(uci get nodered.@nodered[0].password 2>/dev/null)

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  if [ -z "$config" ]; then
    echo "config path is empty!"
    exit 1
  fi

  [ -z "$port" ] && port=1880
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME="nodered/node-red:latest"

  rm -R /opt/docker2/compose/nodered-app
  mkdir /opt/docker2/compose/nodered-app

  touch /opt/docker2/compose/nodered-app/docker-compose.yml
  cat > /opt/docker2/compose/nodered-app/docker-compose.yml <<EOF
version: '3.7'
services:
  nodered:
    image: $IMAGE_NAME
    restart: always
    ports:
      - ${LAN_IP}:$port:1880
    labels:
      plugsy.name: "Node-RED"
      plugsy.category: "Automation"
      plugsy.link: "http://${LAN_IP}:$port"
EOF

  docker-compose -f /opt/docker2/compose/nodered-app/docker-compose.yml up -d

  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$APP_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install Node-RED"
  echo "      upgrade                Upgrade Node-RED"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart Node-RED"
  echo "      status                 Node-RED status"
  echo "      port                   Node-RED port"
}

case ${ACTION} in
  "install"|"upgrade")
    do_install_detail
  ;;
  "rm")
    APP_NAME="nodered"
    IMAGE_NAME=$(uci get nodered.@nodered[0].image_name 2>/dev/null)
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${APP_NAME}" --format '{{.ID}}')
    echo "Stopping and removing containers..."
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
    rm -R /opt/docker2/compose/nodered-app
  ;;
  "start"|"stop"|"restart")
    APP_NAME="nodered"
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${APP_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    APP_NAME="nodered"
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${APP_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    APP_NAME="nodered"
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${APP_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac